#include "spi_flash.h"
#include "nrf_drv_spi.h"
#include "nrf_gpio.h"

static const nrf_drv_spi_t m_spi_mx = NRF_DRV_SPI_INSTANCE(0);

static uint32_t spi_config(void)
{
    uint32_t err_code;
    nrf_drv_spi_config_t config = NRF_DRV_SPI_DEFAULT_CONFIG;
    config.frequency = NRF_DRV_SPI_FREQ_1M;
    config.mode      = NRF_DRV_SPI_MODE_3;
    config.bit_order = NRF_DRV_SPI_BIT_ORDER_MSB_FIRST;
    config.mosi_pin  = MX_PIN_MOSI;
    config.miso_pin  = MX_PIN_MISO;
    config.sck_pin   = MX_PIN_SCK;
    config.ss_pin    = 0xFF; 
    err_code = nrf_drv_spi_init(&m_spi_mx, &config, NULL, NULL);
    if (err_code != NRF_SUCCESS)
    {
        // Initialization failed. Take recovery action.
    }   
    nrf_gpio_cfg_output(MX_PIN_CS);
    nrf_gpio_pin_set(MX_PIN_CS);
    return err_code;
}


uint32_t mx_init(void)
{
    return spi_config();
}


void mx_read_rems(uint8_t * manufacturer_id, uint8_t * device_id)
{
    uint8_t spi_tx_cmd[] = {CMD_REMS, 0x00, 0x00, CMD_REMS_ADDRESS_DEFAULT, 0x00, 0x00};
    uint8_t spi_rx_response[6];
    nrf_gpio_pin_clear(MX_PIN_CS);
    nrf_drv_spi_transfer(&m_spi_mx, spi_tx_cmd, sizeof(spi_tx_cmd), spi_rx_response, sizeof(spi_rx_response));
    nrf_gpio_pin_set(MX_PIN_CS);
    *manufacturer_id = spi_rx_response[4];
    *device_id = spi_rx_response[5];
}

uint8_t mx_read_status_register(void)
{
    uint8_t spi_tx_cmd[2] = {CMD_RDSR, 0};
    static uint8_t spi_rx_rsp[2];
    nrf_gpio_pin_clear(MX_PIN_CS);
    nrf_drv_spi_transfer(&m_spi_mx, spi_tx_cmd, sizeof(spi_tx_cmd), spi_rx_rsp, sizeof(spi_rx_rsp));
    nrf_gpio_pin_set(MX_PIN_CS);
    return  spi_rx_rsp[1];
}


uint8_t mx_read_config_register(void)
{
    uint8_t spi_tx_cmd[2] = {CMD_RDCR, 0};
    static uint8_t spi_rx_rsp[2];
    nrf_gpio_pin_clear(MX_PIN_CS);
    nrf_drv_spi_transfer(&m_spi_mx, spi_tx_cmd, sizeof(spi_tx_cmd), spi_rx_rsp, sizeof(spi_rx_rsp));
    nrf_gpio_pin_set(MX_PIN_CS);
    return  spi_rx_rsp[1];
}


void mx_read(uint32_t address, uint8_t * data_ptr, uint32_t data_length)
{
    uint8_t spi_tx_cmd[] = {CMD_READ, (address >> 16) & 0xFF, (address >> 8) & 0xFF, (address >> 0) & 0xFF};
    nrf_gpio_pin_clear(MX_PIN_CS);
    nrf_drv_spi_transfer(&m_spi_mx, spi_tx_cmd, sizeof(spi_tx_cmd), 0, 0);
    while(data_length > 255)
    {
        nrf_drv_spi_transfer(&m_spi_mx, 0, 0, data_ptr, 255);
        data_ptr += 255;
        data_length -= 255;
    }
    nrf_drv_spi_transfer(&m_spi_mx, 0, 0, data_ptr, data_length);
    nrf_gpio_pin_set(MX_PIN_CS);
}


void mx_write_enable(void)
{
    static uint8_t spi_tx_cmd[] = {CMD_WREN};
    nrf_gpio_pin_clear(MX_PIN_CS);
    nrf_drv_spi_transfer(&m_spi_mx, spi_tx_cmd, sizeof(spi_tx_cmd), 0, 0);
    nrf_gpio_pin_set(MX_PIN_CS);
}

void mx_program(uint32_t address, uint8_t * data_ptr, uint32_t data_length)
{
    uint8_t spi_tx_cmd[] = {CMD_PROGRAM, (address >> 16) & 0xFF, (address >> 8) & 0xFF, (address >> 0) & 0xFF};
    nrf_gpio_pin_clear(MX_PIN_CS);
    nrf_drv_spi_transfer(&m_spi_mx, spi_tx_cmd, sizeof(spi_tx_cmd), 0, 0);
    while(data_length > 255)
    {
        nrf_drv_spi_transfer(&m_spi_mx, data_ptr, 255, 0, 0);
        data_ptr += 255;
        data_length -= 255;
    }
    nrf_drv_spi_transfer(&m_spi_mx, data_ptr, data_length, 0, 0);
    nrf_gpio_pin_set(MX_PIN_CS);
}


void mx_sector_erase(uint32_t address)
{
    uint8_t spi_tx_cmd[] = {CMD_SECTOR_ERASE, (address >> 16) & 0xFF, (address >> 8) & 0xFF, (address >> 0) & 0xFF};
    nrf_gpio_pin_clear(MX_PIN_CS);
    nrf_drv_spi_transfer(&m_spi_mx, spi_tx_cmd, sizeof(spi_tx_cmd), 0, 0);
    nrf_gpio_pin_set(MX_PIN_CS);
}


void mx_chip_erase(void)
{
    static uint8_t spi_tx_cmd[] = {CMD_CHIP_ERASE};
    nrf_gpio_pin_clear(MX_PIN_CS);
    nrf_drv_spi_transfer(&m_spi_mx, spi_tx_cmd, sizeof(spi_tx_cmd), 0, 0);
    nrf_gpio_pin_set(MX_PIN_CS);
}
